/*
 * ============================================================================
 * Licensed Materials - Property of IBM
 *  WebSphere sMash
 *
 * (C) Copyright IBM Corp. 2007  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 * ============================================================================
 */


// 
// File: /app/resources/files.groovy
// 
// Author: Dan Jemiolo (danjemiolo)
// 
// This script includes the RESTful API for the IDE's file system, which has 
// a logical root defined at /config/ide/home.
//

import zero.core.etags.Etag;
import zero.core.fileserver.FileServer;
import zero.ide.services.ExtensionFileFilter;

import zero.appbuilder.resources.MessageHandler

//
// createChildren() returns a list with the names of all sub-directories 
// and files in a given directory. If the target is not a directory, the 
// method returns null.
//
def createChildren(target, filter)
{
    if (!target.isDirectory())
        return null;
    
    def children = target.listFiles(filter);    
    def names = [];
    
    children.each() {
        names << it.getName();
    }
    
    return names;
}

//
// createMetadata() returns a map with metadata about a single file, including 
// its name, path, last-modified date, and a list of any children. If the 
// target is not a directory, the list of children will be null.
//
def createMetadata(target, path)
{
    def isDirectory = target.isDirectory();
    if (isDirectory && !path.endsWith(File.separator))  {
    	// Add a trailing file separator if one isn't there
    	path = path + File.separator;
    }
    
    return [
        name: target.getName(), 
        path: path, 
        modified: target.lastModified(), 
        directory: isDirectory, 
        children: isDirectory ? [] : null 
        //virtual: false
    ];
}

//
// createMetadata() returns a set of metadata for a file or directory tree.
//
def createMetadata(target, path, filter, flat, recurse)
{
    def metadata = createMetadata(target, path);
    
    //
    // if the ?flat param is used, we're going to store all metadata entries 
    // in a single list instead of a tree. we wrap the original metadata map 
    // in a list and then add the metadata of child files/dirs to that list
    //
    if (flat)   
    	{}
    //    metadata = [metadata];   
    
    //
    // otherwise, if we're not flat *and* we're not recursing, we need to 
    // populate the 'children' field with the names of the child files/dirs 
    // so that clients can traverse the tree manually
    //
    
    else if (!recurse)
        metadata.children = createChildren(target, filter);
    
    if (target.isDirectory() && recurse)
    {
        def children = target.listFiles(filter);
        
        children.each() {
            def childMetadata = createMetadata(it, "${path}/${it.getName()}", filter, flat, recurse);
            
            if (flat)
                metadata.addAll(childMetadata);
            
            else
                metadata.children << childMetadata;
        }
    }
    
    return metadata;
}

//
// filterMetadata() removes directories from a set of metadata if the ?nodirs 
// parameter is used; it removes files if ?nofiles is used. If both parameters 
// are used, it returns an empty list.
//
def filterMetadata(metadata)
{
    def noDirs = zcontains("/request/params/nodirs");
    def noFiles = zcontains("/request/params/nofiles");
    
    //
    // remove directories from ?flat listing
    //
    if (noDirs)
    {
        def filtered = [];
        
        metadata.each() {
            if (!it.directory)
                filtered << it;
        }
        
        metadata = filtered;
    }
    
    //
    // remove files from ?flat listing
    //
    if (noFiles)
    {
        def filtered = [];
        
        metadata.each() {
            if (it.directory)
                filtered << it;
        }
        
        metadata = filtered;
    }
    
    return metadata;
}


//
// getETag() returns the ETag for a file as an unquoted string.
//
def getETag(target)
{
    def modified = target.lastModified().toString();
    return Etag.computeHashedValue(modified);
}

//
// getTarget() returns the physical file or directory represented by the 
// request path. If the target does not exist, the method returns null.
//
//def getTarget()
//{
//    def app = getApplication(request.params.applicationsId[]);
//    
//    if (app == null)
//        return null;
//    
//    def appHome = new File(app.path, app.name);
//    
//    if (request.params.filesId[] == null)
//       return appHome;
//    
//    def target = new File(appHome, request.params.filesId[]);
//    return event.pathInfo[] == null ? target : new File(target, event.pathInfo[]);
//}


def getTargets(rootTarget)
{
    def targets = [];
	def children = rootTarget.listFiles();
	
    targets.add(rootTarget);	
	return targets;
}


//
// makeVirtual() traverses a tree of metadata and sets the 'virtual' property 
// on each node to the given value (true or false).
//
//def makeVirtual(target, virtual)
//{
//    target.virtual = virtual;
//    
//    target.children?.each() {
//        makeVirtual(it, virtual);
//    }
//}

//
// mergeMetadata() copies all file and directory metadata entries into a single 
// tree of metadata, taking care to remove duplicate directories and to mark 
// those files and directories that belong to dependencies as 'virtual'.
//
//def mergeMetadata(current, sibling)
//{
//    def childNames = [];
//    
//    
//     create a list of child names for easy comparison below
//    
//    current.children.each() {
//        childNames << it.name;
//    }
//    
//    
//     if a sibling directory has a unique file or directory, 
//     add it to the big tree
//    
//    sibling.children.each() {
//        if (!it.directory || !childNames.contains(it.name))
//        {
//            makeVirtual(it, true);
//            current.children << it;
//        }
//    }
//    
//    
//     move down to the next level in the tree to merge files  
//     in shared directories
//    
//    current.children.each() { child -> 
//        if (!child.virtual)
//        {
//            def matchingChild = null;
//            
//            sibling.children.each() { 
//               if (child.name.equals(it.name))
//                   matchingChild = it;
//            }
//            
//            if (matchingChild != null)
//                mergeMetadata(child, matchingChild);
//        }
//    }
//}

//
// renderDirectory() returns an directory index page to the client.
//
def renderDirectory(dir)
{	
    request.view = "directory.groovy";
    request.json.output = dir;
    render();
}

//
// renderFile() returns the file's contents to the client.
//	
def renderFile(file)
{
    event.path = file.getCanonicalPath();
    def server = new FileServer();
    server.onGET();
}

//
// renderMetadata() returns a set of file metadata to the client.
//
def renderMetadata(target)
{
    def flat = zcontains("/request/params/flat");
    def recurse = zcontains("/request/params/recurse");
    //def virtual = zcontains("/request/params/virtual");
    
    def extensions = request.params.extensions[]?.tokenize(",");
    def filter = new ExtensionFileFilter(extensions, recurse);
    
    //def metadata = createMetadata(target, request.path[], filter, flat, recurse);
    def result = [];
    target.each() {    	
    	def pathData = createMetadata(it, it.toString(), filter, flat, recurse);
    	result.add(pathData);
    }	
    
    if (flat)
        result = filterMetadata(result);
    
    request.view = "JSON";
    request.json.output = result;
    render();
}

//
// REST API begins here!
//

/**
 *
 * @success 200 Returns an index for the IDE's logical root directory.
 * @format text/html
 *
 */
def onList()
{
    onRetrieve();
}

/**
 *
 * @success 200 Returns the contents of the file or the index of a directory.
 * @error 404 If the file or directory does not exist.
 * @format any content type
 * 
 */
def onRetrieve()
{
    def root = zget("/request/params/root");

    def targets;
    if ((root) && (root.length() >0))
    {       
        targets = [];
    	def rootTarget = new File(root);
    	if (!rootTarget.exists()) {    		
    	    request.status = 404;
    	    return;
    	}    		
        
        targets.add(rootTarget);
// 		Return all of the parent directories of the root
//        def parent = rootTarget.getParentFile();
//        while (parent) {
//        	targets.add(parent);
//        	parent = parent.getParentFile();
//        }    	
    	// Also return a list of all of the children of the root directory
    	def children = rootTarget.listFiles();
    	children.each() {    		
        	targets.add(it);
    	}
    }
    else    
    {
        targets = File.listRoots();        
    }
    
    renderMetadata(targets);
}



